/*
 * sign.c: a program to present text on a TCP port
 *
 * Author: Jeremy Elson (jelson@cs.jhu.edu)
 *  Usage: sign <port> <filename> or
 *         sign <port> -
 *
 * '-' indicates file should be read from stdin.
 *
 * This program is in the public domain.  It may be copied, redistributed,
 * reused, modified, etc., but a notice of my authorship must be maintained.
 *
 * This program comes with no warranty of any kind, expressed or implied.
 */
/*
 * WinSign - based on sign.c (see above for info)
 *
 * Made Windows-friendly by Joe DiLascio
 */
#define MAX_FILESIZE	8192
#define LINEBUF_SIZE	128

// Change below if not 2 directories below 'src'
#include "..\..\conf.h"
#include "..\..\sysdep.h"

/*
 * init_socket sets up the mother descriptor - creates the socket, sets
 * its options up, binds it, and listens.
 */
int init_socket(int port)
{
	int s, opt;
	struct sockaddr_in sa;

	{
		WORD wVersionRequested;
		WSADATA wsaData;

		wVersionRequested = MAKEWORD(1, 1);

		if (WSAStartup(wVersionRequested, &wsaData) != 0) {
			fprintf(stderr, "SYSERR: WinSock not available!\n");
			exit(1);
		}

		if ((s = socket(PF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
			fprintf(stderr, "SYSERR: Error opening network connection: Winsock error #%d\n", WSAGetLastError());
			exit(1);
		}
	}

	opt = 1;
	if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &opt, sizeof(opt)) < 0) {
		perror("SYSERR: setsockopt REUSEADDR");
		exit(1);
	}

	{
		struct linger ld;

		ld.l_onoff = 0;
		ld.l_linger = 0;
		if (setsockopt(s, SOL_SOCKET, SO_LINGER, (char *) &ld, sizeof(ld)) < 0) {
			perror("SYSERR: setsockopt LINGER");
			exit(1);
		}
	}

	// Clear the structure
	memset((char *)&sa, 0, sizeof(sa));

	sa.sin_family = AF_INET;
	sa.sin_port = htons(port);
	sa.sin_addr.s_addr = htonl(INADDR_ANY);

	if (bind(s, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
		perror("SYSERR: bind");
		CLOSE_SOCKET(s);
		exit(1);
	}
	listen(s, 5);
	return s;
}

char *get_text(char *fname)
{
	static char t[MAX_FILESIZE];
	char tmp[LINEBUF_SIZE + 2];
	FILE *fl = NULL;

	*t = '\0';

	if (!strcmp(fname, "-")) {
		fl = stdin;
		fprintf(stderr, "Enter sign text; terminate with @ on a new line.\n");
	} else {
		if (!(fl = fopen(fname, "r"))) {
			perror(fname);
			exit(1);
		}
	}

	while (fgets(tmp, LINEBUF_SIZE, fl)) {
		if (tmp[0] == '@') {
			fprintf(stderr, "Ok, got sign.\n\n");
			break;
		}
		if (strlen(tmp) + strlen(t) < MAX_FILESIZE - 1)
			strcat(t, strcat(tmp, "\r"));
		else {
			fprintf(stderr, "String too long.  Truncated.\n");
			break;
		}
	}

	return t;
}

void main(int argc, char *argv[])
{
	char *txt, *tmp;
	int desc, remaining, bytes_written, len, s, port;

	if (argc != 3 || (port = atoi(argv[1])) < 1024) {
		fprintf(stderr, "usage: %s <portnum> <\"-\" | filename>\n", argv[0]);
		exit(1);
	}
	s = init_socket(port);
	len = strlen(txt = get_text(argv[2]));

	fprintf(stderr, "Sign started on port %d.\n", port);
	fprintf(stderr, "CTRL-C to end.\n");

	tmp = txt;
	for (;;) {
		if ((desc = accept(s, (struct sockaddr *) NULL, 0)) < 0)
			continue;

		remaining = len;
		txt = tmp;
		do {
			if ((bytes_written = send(desc, txt, remaining, 0)) < 0) {
				fprintf(stderr, "Error: send()... closing\n");
				closesocket(desc);
			} else {
				txt += bytes_written;
				remaining -= bytes_written;
			}
		} while (remaining > 0);

		closesocket(desc);
	}
}
